package com.lovecoding.aop;

import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.*;
import org.springframework.stereotype.Component;

@Aspect
@Component
public class CounterProxy {

    /**
     * 接收通知的方法
     * Before 前置通知
     * After 后置通知
     * AfterReturning 返回通知
     * Around  环绕通知
     * AfterThrowing 异常通知
     */
    @Before("execution(* com.lovecoding.aop.CountterImpl.*(..))")
    public void add(JoinPoint joinPoint){
        System.out.println( "前置通知" );
    }

    @After("execution(public int com.lovecoding.aop.CountterImpl.*(..))")
    public void AfterAdd(JoinPoint joinPoint){
        Signature signature = joinPoint.getSignature();
        String name = signature.getName();
        //System.out.println( "切入点 方法" + name );
        System.out.println( "后置通知" );
    }

    @AfterReturning("execution(public int com.lovecoding.aop.CountterImpl.*(..))")
    public void afterReturning(JoinPoint joinPoint){
        System.out.println( "返回通知" );
    }

    @Around("execution(public int com.lovecoding.aop.CountterImpl.*(..))")
    public Object around( ProceedingJoinPoint joinPoint ){
        try {
            System.out.println( "环绕通知前" );
            Object proceed = joinPoint.proceed(joinPoint.getArgs());
            System.out.println( "环绕通知后" );
            return proceed;
        } catch (Throwable e) {
            e.printStackTrace();
        }
        return null;
    }

    @AfterThrowing(value = "execution(public int com.lovecoding.aop.CountterImpl.*(..))", throwing = "e")
    public void AfterThrowing(JoinPoint joinPoint, Throwable e){
        Signature signature = joinPoint.getSignature();
        String name = signature.getName();
        System.out.println( "方法 ：" + name + "发生异常" );
        System.out.println( e.getMessage() );
    }

}
